#!/usr/bin/env python3
from contextlib import contextmanager


class GraphDatabase:

    def __init__(self, storage):
        self._storage = storage

    def close(self):
        self._storage.close()

    def txn(self, sync=True):
        """Return a storage specific transaction.

        See each storage for further informations regarding the keywords
        arguments. Here is a brief overview what can be supported:

        ``sync`` must be true, if you want transactions to be immediatly sync
        to disk changes made to the database. Setting ``sync=False`` will
        provide a speed boost you might loose data, but not corrupt
        your database.
        """
        return self._storage._txn(sync)

    @contextmanager
    def transaction(self, **kwargs):
        """Context manager to start a new transaction.

        See ``GraphDatabase.txn()`` to know the meaning of ``kwargs``"""
        txn = self.txn(**kwargs)
        try:
            yield txn
        except Exception as exc:
            txn.rollback()
            raise exc
        else:
            txn.commit()
