# Welcome to AjguDB Documentation

## Kesako?!


**AjguDB** is graph database written in Python with several backends. It's meant to be an easy to use,
just works persistent graph for people that want to experiment with graph databases. SQLite for graphs,
if I may.


## Getting started

### Create a Graph

You need to choose a storage class and create a graph. There is three storages right now:

- `ajgu.storage.BSDDB3` the default storage which use Oracle Berkeley Database 5.x or 6.x.
- `ajgu.storage.NoTxnBSDDB3` a bsddb3 backend than doesn't use transactiosn at hence it not safe.
- `ajgu.storage.FileStorage` a toy file based backend.

You can start graphology as easily as:

```python
from ajgu import Database
from ajgu import BSDDB3


graph = Database(BSDDB3('/path/to/my/super-dupper/database/files'))
```
If the directory doesn't exists it will be created.

Remember to close the database, when you are finished::

```python
graph.close()
```

### The Mighty Transaction

Every operation must happen in a transaction. For that matter there is context manager `Graph.transaction()`
that return a storage specific transaction class. It must be used as follow:

```python
with graph.transaction() as txn:
    txn.vertex.create('my-vertex')
    ...
    ...
```

The transaction will be commited if there is no error.

For advanced use, you can use ``Graph.txn()`` method to create a transaction and manage it yourself. It looks
like this:

```python
try:
    txn = graph.txn()
    txn.create_vertex('my-vertex')
    ...
    ...
except Exception as exc:
    txn.rollback()
else:
    txn.commit()
```

`commit` and `rollback` are the only methods of transaction class. Each backend provides it's own transaction class
that must implement those methods. 

Both ``Graph.transaction`` and ``Graph.txn`` take keyword arguments to configure the transaction. It depends of the backend.

### Create a vertex

The transaction object works as a *shadow copy* of the graph. You always perform graph operations against it,
Once the transaction is commited, changes are persisted are saved.

To create a vertex using `Transaction.vertex.create(label)`:

```python
with graph.transaction() as txn:
    amirouche = txn.vertex.create("amirouche")
    amirouche.properties['age'] = 30
```

As you can see in the above snipped it's possible to add properties to a vertex through ``Vertex.properties`` just like you do with a normal dictionary. `Vertex` are indexed by their `Vertex.label()`. Several vertices can have the same label.

### Create an edge

To create an edge you can use `Transaction.edge.create(start, label, end)`:

```python
with graph.transaction() as txn:
    amirouche = txn.vertex.create("amirouche")
    python = txn.vertex.create("python")
    txn.edge.create(amirouche 'know', python)
```

`Edge` have an index on their label, also available as `Edge.label()`. labels doesn't have be unique. `start` and `end` must be vertices created or retrieve in the current transaction. You can also edit edge's properties via `Edge.properties` attribute.
    

### Transaction.*.get

You can retrieve edge with `Transaction.edge.get(identifier)` and vertices with `Transaction.vertex.get(identifier`.
 
### Vertices outgoing and incoming edges


Vertex has ``Vertex.outgoings(label=None)`` and ``Vertex.incomings(label=None)``. They allow to retrieve edges. The label argument allows to filter based on the ``edge.label()`` of the edge. They return a generator. They will be explained further below.
