import sys
from datetime import datetime
from ajgu import Storage
from ajgu import GraphDatabase

import msgpack

progress = 'checkpoint {} {:f}%'


def main():
    storage = Storage('./_db', (3, 0), (4, 0))
    graph = GraphDatabase(storage)
    
    keys = [
        'sources',
        'surfaceText',
        'source_uri',
        'dataset',
        'id',
        'uri',
        'features',
        'context',
        'weight',
        'license'
    ]
    STEP = datetime.now()
    EDGE_COUNT = 0
    TOTAL_EDGE = 13280680
    try:
        print('Starting load in `_db/`')
        print('This takes several hours on an average hardware!')
        index = sys.argv[1]
        name = './part_0{}.msgpack'.format(index)
        with open(name, 'rb') as stream:
            unpacker = msgpack.Unpacker(stream, encoding='utf-8')
            for num, value in enumerate(unpacker):
                with graph.transaction(sync=False) as txn:
                    start = list(txn.vertex.label(value['start']).all())
                    if start:
                        start = start[0]
                    else:
                        start = txn.vertex.create(value['start'])
                    end = list(txn.vertex.label(value['end']).all())
                    if end:
                        end = end[0]
                    else:
                        end = txn.vertex.create(value['end'])
                    relation = txn.edge.create(start, value['rel'], end)
                    for key in keys:
                        relation.props[key] = value[key]
                    EDGE_COUNT += 1
                    if EDGE_COUNT % 10000 == 0:
                        print(EDGE_COUNT, datetime.now() - STEP)
                        graph._storage.checkpoint()
                        STEP = datetime.now()
        print(progress.format(name, TOTAL_EDGE))
        graph._storage.checkpoint()
    except Exception as exc:
        print(exc)
    print('closing')
    graph.close()
    sys.exit()


if __name__ == '__main__':
    main()
