import os

from jinja2 import Environment
from jinja2 import FileSystemLoader


class Jinja:

    def __init__(self, *paths, **filters):
        paths = map(os.path.abspath, paths)
        self.environment = Environment(
            loader=FileSystemLoader(paths),
        )
        self.environment.filters = filters

    def __call__(self, template, **context):
        template = self.environment.get_template(template)
        out = template.render(**context)
        return out


def render(template, *paths, **context):
    filters = dict([e for e in context.items() if callable(e[1])])
    jinja = Jinja(*paths, **filters)
    output = jinja(template, **context)
    return output
