import asyncio
from time import time
from pathlib import Path

from aiohttp import web
from aiohttp.web import HTTPSeeOther

from ajgu import GraphDatabase
from ajgu.storage.file import FileStorage as Storage

from render import render as _render


ROOT = Path('.').resolve()
DB_ROOT = str(ROOT / 'db')
TEMPLATE_ROOT = str(ROOT / 'templates')

storage = Storage(DB_ROOT)
db = GraphDatabase(storage)

render = lambda tmpl, **ctx: _render(tmpl, TEMPLATE_ROOT, **ctx)


@asyncio.coroutine
def index(request):
    with db.transaction() as txn:
        messages = reversed(sorted(txn.vertex.all(), key=lambda x: x.props.created_at))
    text = render('index.jinja', messages=messages)
    return web.Response(body=text.encode('utf-8'))


@asyncio.coroutine
def post(request):
    data = yield from request.post()
    with db.transaction() as txn:
        vertex = txn.vertex.create('message')
        vertex.properties['body'] = data['message']
        vertex.properties['created_at'] = time()
    return HTTPSeeOther('/')


@asyncio.coroutine
def init(loop):
    app = web.Application(loop=loop)
    app.router.add_static('/static', str(ROOT / 'static'))
    app.router.add_route('GET', '/', index)
    app.router.add_route('POST', '/post', post)
    srv = yield from loop.create_server(
        app.make_handler(),
        '0.0.0.0',
        8080
    )
    print("Server started at http://0.0.0.0:8080")
    return srv


loop = asyncio.get_event_loop()
loop.run_until_complete(init(loop))
try:
    loop.run_forever()
except KeyboardInterrupt:
    db.close()
