import sys
from csv import reader

from ajgu import Storage
from ajgu import GraphDatabase

import msgpack


progress = 'checkpoint {} {:f}%'


def main():
    storage = Storage('./_db', (4, 0), (8, 0))
    graph = GraphDatabase(storage)

    print('Starting load of `movies.csv` in `_db/`')
    with open('./ml-latest-small/movies.csv', 'r') as f:
        next(reader(f))
        with graph.transaction(sync=False) as txn:
            for uid, title, genres in reader(f):
                genres = genres.split('|')
                movie = txn.vertex.label('movie:' + uid).first()
                if not movie:
                    movie = txn.vertex.create('movie:' + uid)
                movie.props['title'] = title
                movie.props['genres'] = genres
    print('Starting load of `ratings.csv` in `_db/`')
    with open('./ml-latest-small/ratings.csv', 'r') as f:
        next(reader(f))
        INDEX = 0
        from datetime import datetime
        NOW = datetime.now()
        for userid, movieid, rating, timestamp in reader(f):
            INDEX += 1
            if INDEX % 10000 == 0:
                print(INDEX, datetime.now() - NOW)
                graph._storage.checkpoint()
                NOW = datetime.now()
            with graph.transaction(sync=False) as txn:
                movie = txn.vertex.label('movie:' + movieid).first()
                user = txn.vertex.label('user:' + userid).first()
                if not user:
                    user = txn.vertex.create('user:' + userid)
                edge = txn.edge.create(user, 'rating', movie)
                edge.props['value'] = float(rating)
                edge.props['timestamp'] = float(timestamp)
    print('Starting load of `tags.csv` in `_db/`')
    with open('./ml-latest-small/tags.csv', 'r') as f:
        next(reader(f))
        with graph.transaction(sync=False) as txn:
            for userid, movieid, tag, timestamp in reader(f):
                movie = txn.vertex.label('movie:' + movieid).first()
                user = txn.vertex.label('user:' + userid).first()
                edge = txn.edge.create(user, 'tag', movie)
                edge.props['value'] = tag
                edge.props['timestamp'] = float(timestamp)
    print('closing')
    graph.close()
    sys.exit()


if __name__ == '__main__':
    main()
