from re import compile
from json import loads
from pathlib import Path
from base64 import b64decode

from ajgu import Storage
from ajgu import GraphDatabase


print('* opening database')
path = str(Path('./_db').absolute())
storage = Storage(str(path), (1, 0),  (2, 0))
db = GraphDatabase(storage)

# cache identifiers to speed things up
NAME_TO_ID = dict()

NAME = compile('^([\w\.]+)')


def get_or_create(txn, name):
    try:
        identifier = NAME_TO_ID[name]
    except KeyError:
        vertex = list(txn.vertex.label(name))
        if vertex:
            vertex = vertex[0]
            NAME_TO_ID[name] = vertex.identifier()
            return vertex
        # not found, create it
        vertex = txn.vertex.create(name)

        # retrieve json data from pypi
        vertex.props['origin'] = 'pypi'
        NAME_TO_ID[name] = vertex.identifier()
        return vertex
    else:
        return txn.vertex.get(identifier)

# the pypi dataset was built by O. Girardot
# https://ogirardot.wordpress.com/2013/01/31/sharing-pypimaven-dependency-data/
with Path('./load.log').open('w') as log:
    with Path('./pypi-deps.csv').resolve().open() as f:
        print('* opening ./pypi-deps.csv')
        lines = f.read().split('\n')
        count = len(lines)
        print('* loading...')
        print('* check tail -f ./load.log')
        for num, line in enumerate(lines):
            with db.transaction(sync=False) as txn:
                name, version, dependencies = line.split('\t')
                dependencies = loads(b64decode(dependencies).decode('utf-8'))
                software = get_or_create(txn, name)

                dependencies = map(lambda x: x.strip().lower(), dependencies)
                dependencies = list(dependencies)

                # several package miss the dependency
                if 'django' in name and name != 'django':
                    if 'django' not in dependencies:
                        dependencies.append('Django')

                for dependency in dependencies:
                    if 'Extra requirements' in dependency:
                        continue
                    if dependency.startswith('#'):
                        continue
                    if 'requirements.txt' in dependency:
                        continue
                    if dependency == '-e))':
                        continue

                    if dependency.startswith('"') and dependency.endswith('"'):
                        dependency = dependency[1:-1]

                    dependency = NAME.search(dependency)
                    if not dependency:
                        continue
                    dependency = dependency.group()
                    dependency = get_or_create(txn, dependency)
                    edge = txn.edge.create(software, 'depends', dependency)
                    edge.props['name'] = software.label()
                    log.write('{}% @ {}\n'.format((num / count) * 100, name))

db.close()
